package com.hz.pm.api.common.util;

import cn.hutool.core.util.ReflectUtil;
import cn.hutool.core.util.StrUtil;

import java.lang.reflect.Method;
import java.util.function.Function;

/**
 * <p>
 * MethodReferenceUtil
 * </p>
 *
 * @author WendyYang
 * @since 16:24 2024/8/6
 */
public class MethodReferenceUtil {

    private MethodReferenceUtil() {
    }

    /**
     * 根据属性名获取方法引用。
     *
     * @param field 属性名
     * @return 方法引用
     */
    public static <T> Function<T, Object> getReference(Class<T> clazz, String field) {
        // 将属性名首字母大写
        String methodName = StrUtil.upperFirst(field);
        Method method = ReflectUtil.getMethod(clazz, "get" + methodName);
        if (method == null) {
            return null;
        }
        return obj -> ReflectUtil.invoke(obj, method);
    }

}
