package com.hz.pm.api.projectlib.helper;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.core.toolkit.support.SFunction;
import com.hz.pm.api.common.model.constant.BizConst;
import com.hz.pm.api.common.statemachine.event.ProjectStateChangeEvent;
import com.hz.pm.api.common.statemachine.event.TenderStateChangeEvent;
import com.hz.pm.api.common.util.MethodReferenceUtil;
import com.hz.pm.api.projectdeclared.entity.ProjectChangeHistory;
import com.hz.pm.api.projectdeclared.model.dto.*;
import com.hz.pm.api.projectdeclared.model.entity.Contract;
import com.hz.pm.api.projectdeclared.model.entity.Operation;
import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectdeclared.service.IContractService;
import com.hz.pm.api.projectdeclared.service.IOperationService;
import com.hz.pm.api.projectdeclared.service.IProjectChangeHistoryService;
import com.hz.pm.api.projectdeclared.service.IPurchaseService;
import com.hz.pm.api.projectdeclared.utils.BeanDiffUtil.FieldDiff;
import com.hz.pm.api.projectlib.entity.PurchaseStatusChange;
import com.hz.pm.api.projectlib.manage.ProjectGovSystemReplaceInfoManage;
import com.hz.pm.api.projectlib.manage.ProjectLibManage;
import com.hz.pm.api.projectlib.model.dto.ProjectDTO;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.entity.ProjectStatusChange;
import com.hz.pm.api.projectlib.model.enumeration.status.*;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.hz.pm.api.projectlib.service.IProjectStatusChangeService;
import com.hz.pm.api.projectlib.service.IPurchaseStatusChangeService;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.basic.util.CollUtils;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.function.Function;

/**
 * <p>
 * ProjectChangeStopHelper
 * </p>
 *
 * @author WendyYang
 * @since 14:37 2024/8/5
 */
@Component
@RequiredArgsConstructor
public class ProjectChangeStopHelper {

    private final IProjectChangeHistoryService projectChangeHistoryService;
    private final IPurchaseStatusChangeService purchaseStatusChangeService;
    private final IProjectStatusChangeService projectStatusChangeService;
    private final IPurchaseService purchaseService;
    private final IOperationService operationService;
    private final IContractService contractService;
    private final IProjectService projectService;
    private final ProjectGovSystemReplaceInfoManage projectGovSystemReplaceInfoManage;
    private final ProjectLibManage projectLibManage;

    /**
     * 根据状态变更记录重置项目、标段状态
     *
     * @param project 项目
     * @param event   event
     */
    public void changeProjectPurchaseStatus(Project project, ProjectStateChangeEvent event) {
        TenderStateChangeEvent tenderEvent = TenderStateChangeEvent.valueOf(event.name());
        ProjectStateChangeEvent beforeEvent;
        TenderStateChangeEvent tenderBeforeEvent;
        if (event.equals(ProjectStateChangeEvent.STOPPED_APPLY_FAILED)) {
            beforeEvent = ProjectStateChangeEvent.STOPPED_APPLY_SUBMIT;
            tenderBeforeEvent = TenderStateChangeEvent.STOPPED_APPLY_SUBMIT;
        } else if (event.equals(ProjectStateChangeEvent.CHANGE_APPLY_FAILED)
                || event.equals(ProjectStateChangeEvent.CHANGE_APPLY_PASS)) {
            beforeEvent = ProjectStateChangeEvent.CHANGE_APPLY_SUBMIT;
            tenderBeforeEvent = TenderStateChangeEvent.CHANGE_APPLY_SUBMIT;
        } else if (event.equals(ProjectStateChangeEvent.STOPPED_APPLY_SUBMIT)
                || event.equals(ProjectStateChangeEvent.CHANGE_APPLY_SUBMIT)
                || event.equals(ProjectStateChangeEvent.STOPPED_APPLY_PASS)
                || event.equals(ProjectStateChangeEvent.STOPPED_APPLY_AUTO_PASS)) {
            // 提交的时候不重置项目、标项状态
            beforeEvent = null;
            tenderBeforeEvent = null;
            ProjectStatus status;
            TenderStatus tenderStatus;
            if (event.equals(ProjectStateChangeEvent.STOPPED_APPLY_SUBMIT)) {
                status = ProjectStatus.ON_STOPPED_APPLY;
                tenderStatus = TenderStatus.ON_STOPPED_APPLY;
            } else if (event.equals(ProjectStateChangeEvent.CHANGE_APPLY_SUBMIT)) {
                status = ProjectStatus.ON_CHANGE_APPLY;
                tenderStatus = TenderStatus.ON_CHANGE_APPLY;
            } else {
                project.setStoppedStatus(buildBizStatus(event).getCode());
                status = ProjectStatus.STOPPED_PASSED;
                tenderStatus = TenderStatus.STOPPED_PASSED;
            }
            ProjectStatusChange projectStatusChange = ProjectStatusChange.builder()
                    .event(event.name())
                    .projectId(project.getId())
                    .event(event.name())
                    .projectCode(project.getProjectCode())
                    .beforeStatus(project.getStatus())
                    .afterStatus(status.getCode())
                    .build();
            projectStatusChangeService.save(projectStatusChange);
            project.setStage(project.getStatus() - project.getStatus() % 10000);
            projectService.updateById(project);
            List<Purchase> purchases = purchaseService.listByProjectId(project.getId());
            List<PurchaseStatusChange> purchaseStatusChanges = CollUtils.convert(purchases,
                    purchase -> PurchaseStatusChange.builder()
                            .bidId(purchase.getId())
                            .event(tenderEvent.name())
                            .projectId(project.getId())
                            .projectCode(project.getProjectCode())
                            .beforeStatus(purchase.getStatus())
                            .afterStatus(tenderStatus.getCode())
                            .build());
            purchaseStatusChangeService.saveBatch(purchaseStatusChanges);
        } else {
            throw new BizException("不支持的状态转换");
        }
        resetProjectPurchaseStatus(project, event, beforeEvent, tenderBeforeEvent, tenderEvent);
    }

    private void resetProjectPurchaseStatus(Project project,
                                            ProjectStateChangeEvent projEvent,
                                            ProjectStateChangeEvent projBeforeEvent,
                                            TenderStateChangeEvent tenderBeforeEvent,
                                            TenderStateChangeEvent tenderEvent) {
        if (projBeforeEvent != null) {
            if (projEvent.equals(ProjectStateChangeEvent.STOPPED_APPLY_FAILED)) {
                project.setStoppedStatus(buildBizStatus(projBeforeEvent).getCode());
            } else {
                Wrapper<ProjectChangeHistory> changeHistoryWrapper = Wrappers.lambdaQuery(ProjectChangeHistory.class)
                        .eq(ProjectChangeHistory::getProjectCode, project.getProjectCode())
                        .orderByDesc(ProjectChangeHistory::getCreateOn)
                        .last(BizConst.LIMIT_1);
                ProjectChangeHistory changeHistory = projectChangeHistoryService.getOne(changeHistoryWrapper);
                changeHistory.setStatus(buildBizStatus(projEvent).getCode());
                projectChangeHistoryService.updateById(changeHistory);
                if (projEvent.equals(ProjectStateChangeEvent.CHANGE_APPLY_PASS)) {
                    projectEditRewrite(JSONUtil.toBean(changeHistory.getChangeContent(), ProjectEditDTO.class));
                }
            }
            ProjectStatusChange change = projectStatusChangeService.getLastOne(projBeforeEvent, project.getId());
            projectStatusChangeService.save(ProjectStatusChange.builder()
                    .event(projEvent.name())
                    .projectId(project.getId())
                    .event(projEvent.name())
                    .projectCode(project.getProjectCode())
                    .beforeStatus(change.getAfterStatus())
                    .afterStatus(change.getBeforeStatus())
                    .createOn(project.getCreateOn())
                    .build());
            project.setStatus(change.getBeforeStatus());
            project.setStage(project.getStatus() - project.getStatus() % 10000);
            projectService.updateById(project);
            List<Purchase> purchases = purchaseService.listByProjectId(project.getId());
            if (!purchases.isEmpty()) {
                Wrapper<PurchaseStatusChange> purchaseStatusChangeQuery = Wrappers.lambdaQuery(PurchaseStatusChange.class)
                        .eq(PurchaseStatusChange::getProjectId, project.getId())
                        .eq(PurchaseStatusChange::getEvent, tenderBeforeEvent);
                List<PurchaseStatusChange> purchaseStatusChanges = purchaseStatusChangeService.list(purchaseStatusChangeQuery);
                Map<Long, PurchaseStatusChange> purchaseStatusChangeMap = CollUtils.listToMap(purchaseStatusChanges, PurchaseStatusChange::getBidId);
                List<PurchaseStatusChange> changeArrayList = new ArrayList<>();
                purchases.forEach(purchase -> {
                    PurchaseStatusChange currChange = purchaseStatusChangeMap.get(purchase.getId());
                    purchase.setStatus(currChange.getBeforeStatus());
                    changeArrayList.add(PurchaseStatusChange.builder()
                            .afterStatus(currChange.getBeforeStatus())
                            .beforeStatus(currChange.getAfterStatus())
                            .bidId(purchase.getId())
                            .event(tenderEvent.name())
                            .projectId(project.getId())
                            .projectCode(project.getProjectCode())
                            .build());
                });
                purchaseService.updateBatchById(purchases);
                purchaseStatusChangeService.saveBatch(changeArrayList);
            }
        }
    }

    private IStatus<Integer, String> buildBizStatus(ProjectStateChangeEvent event) {
        IStatus<Integer, String> bizStatus;
        switch (event) {
            case STOPPED_APPLY_PASS:
            case STOPPED_APPLY_AUTO_PASS:
                bizStatus = ProjectStoppedStatus.STOPPED_PASSED;
                break;
            case STOPPED_APPLY_SUBMIT:
                bizStatus = ProjectStoppedStatus.ON_STOPPED_APPLY;
                break;
            case STOPPED_APPLY_FAILED:
                bizStatus = ProjectStoppedStatus.STOPPED_FAILED;
                break;
            case CHANGE_APPLY_SUBMIT:
                bizStatus = ProjectChangeStatus.ON_CHANGE_APPLY;
                break;
            case CHANGE_APPLY_PASS:
                bizStatus = ProjectChangeStatus.CHANGE_PASSED;
                break;
            case CHANGE_APPLY_FAILED:
                bizStatus = ProjectChangeStatus.CHANGE_FAILED;
                break;
            default:
                bizStatus = null;
                break;
        }
        return bizStatus;
    }

    private void projectEditRewrite(ProjectEditDTO projectEdit) {
        List<FieldDiff> projectDiffs = projectEdit.getProjectDiffs();
        if (projectDiffs != null && !projectDiffs.isEmpty()) {
            ProjectDTO projectDetail = projectEdit.getProject();
            LambdaUpdateWrapper<Project> wrapper = Wrappers.lambdaUpdate(Project.class);
            projectDiffs.forEach(fieldDiff -> {
                String fieldName = fieldDiff.getFieldName();
                Object newValue = fieldDiff.getNewValue();
                Function<Project, Object> reference = MethodReferenceUtil.getReference(Project.class, fieldName);
                if (reference != null) {
                    wrapper.set((SFunction<Project, Object>) reference, newValue);
                }
                if (ProjectDTO.SYSTEM_REPLACE_INFOS.equals(fieldName)) {
                    projectLibManage.saveGovSystemReplaceInfos(projectDetail.getId(),
                            projectDetail.getProjectCode(),
                            projectDetail.getSystemReplaceInfos());
                }
            });
            wrapper.eq(Project::getId, projectDetail.getId());
            projectService.update(wrapper);
        }
        List<PurchaseEditDTO> purchases = projectEdit.getPurchases();
        if (CollUtil.isNotEmpty(purchases)) {
            for (PurchaseEditDTO purchaseEdit : purchases) {
                List<FieldDiff> purchaseDiffs = purchaseEdit.getPurchaseDiffs();
                if (CollUtil.isNotEmpty(purchaseDiffs)) {
                    PurchaseDTO purchaseDetail = purchaseEdit.getPurchase();
                    LambdaUpdateWrapper<Purchase> wrapper = Wrappers.lambdaUpdate(Purchase.class);
                    purchaseDiffs.forEach(fieldDiff -> {
                        String fieldName = fieldDiff.getFieldName();
                        Object newValue = fieldDiff.getNewValue();
                        Function<Purchase, Object> reference = MethodReferenceUtil.getReference(Purchase.class, fieldName);
                        if (reference != null) {
                            wrapper.set((SFunction<Purchase, Object>) reference, newValue);
                        }
                    });
                    wrapper.eq(Purchase::getId, purchaseDetail.getId());
                    purchaseService.update(wrapper);
                }
                List<FieldDiff> contractDiffs = purchaseEdit.getContractDiffs();
                if (CollUtil.isNotEmpty(contractDiffs)) {
                    ContractDTO contractDetail = purchaseEdit.getContract();
                    LambdaUpdateWrapper<Contract> wrapper = Wrappers.lambdaUpdate(Contract.class);
                    contractDiffs.forEach(fieldDiff -> {
                        String fieldName = fieldDiff.getFieldName();
                        Object newValue = fieldDiff.getNewValue();
                        Function<Contract, Object> reference = MethodReferenceUtil.getReference(Contract.class, fieldName);
                        if (reference != null) {
                            wrapper.set((SFunction<Contract, Object>) reference, newValue);
                        }
                    });
                    wrapper.eq(Contract::getId, contractDetail.getId());
                    contractService.update(wrapper);
                }
                List<FieldDiff> operationDiffs = purchaseEdit.getOperationDiffs();
                if (CollUtil.isNotEmpty(operationDiffs)) {
                    OperationDTO operationDetail = purchaseEdit.getOperation();
                    LambdaUpdateWrapper<Operation> wrapper = Wrappers.lambdaUpdate(Operation.class);
                    operationDiffs.forEach(fieldDiff -> {
                        String fieldName = fieldDiff.getFieldName();
                        Object newValue = fieldDiff.getNewValue();
                        Function<Operation, Object> reference = MethodReferenceUtil.getReference(Operation.class, fieldName);
                        if (reference != null) {
                            wrapper.set((SFunction<Operation, Object>) reference, newValue);
                        }
                    });
                    wrapper.eq(Operation::getId, operationDetail.getId());
                    operationService.update(wrapper);
                }
            }
        }
    }

}
