package com.hz.pm.api.projectlib.manage;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hz.pm.api.common.enumeration.ProjectProcessType;
import com.hz.pm.api.external.model.enumeration.MhUnitStripEnum;
import com.hz.pm.api.projectdeclared.entity.ProjectChangeHistory;
import com.hz.pm.api.projectdeclared.manage.DeclaredProjectHelper;
import com.hz.pm.api.projectdeclared.model.dto.*;
import com.hz.pm.api.projectdeclared.model.vo.ContractVO;
import com.hz.pm.api.projectdeclared.model.vo.OperationVO;
import com.hz.pm.api.projectdeclared.model.vo.PurchaseFullInfoVO;
import com.hz.pm.api.projectdeclared.model.vo.PurchaseVO;
import com.hz.pm.api.projectdeclared.service.IProjectChangeHistoryService;
import com.hz.pm.api.projectdeclared.utils.BeanDiffUtil;
import com.hz.pm.api.projectdeclared.utils.BeanDiffUtil.FieldDiff;
import com.hz.pm.api.projectlib.helper.ProjectChangeStopHelper;
import com.hz.pm.api.projectlib.helper.ProjectManageUtil;
import com.hz.pm.api.projectlib.model.dto.GovSystemReplaceInfoDTO;
import com.hz.pm.api.projectlib.model.dto.ProjectChangeListDTO;
import com.hz.pm.api.projectlib.model.dto.ProjectDTO;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.entity.ProjectInst;
import com.hz.pm.api.projectlib.model.enumeration.status.ProjectChangeStatus;
import com.hz.pm.api.projectlib.model.enumeration.status.ProjectStatus;
import com.hz.pm.api.projectlib.model.req.ProjectListReq;
import com.hz.pm.api.projectlib.model.req.SubmitProjectChangeApplyReq;
import com.hz.pm.api.projectlib.model.vo.ProjectChangeDetailVO;
import com.hz.pm.api.projectlib.model.vo.ProjectChangeListVO;
import com.hz.pm.api.projectlib.model.vo.ProjectDetailVO;
import com.hz.pm.api.projectlib.model.vo.ProjectLibListItemVO;
import com.hz.pm.api.projectlib.service.IProjectInstService;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.hz.pm.api.sys.manage.ProcessModelManage;
import com.hz.pm.api.user.security.model.UserInfoDetails;
import com.hz.pm.api.user.util.LoginUserUtil;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.basic.model.ITree;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.basic.util.CollUtils;
import com.wflow.bean.entity.WflowModels;
import com.wflow.workflow.bean.dto.OrgInfoDTO;
import com.wflow.workflow.bean.vo.ProcessStartParamsVo;
import com.wflow.workflow.service.ProcessInstanceService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static com.hz.pm.api.common.statemachine.event.ProjectStateChangeEvent.CHANGE_APPLY_SUBMIT;

/**
 * <p>
 * ProjectStoppedManage
 * </p>
 *
 * @author WendyYang
 * @since 11:04 2024/8/5
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class ProjectChangeManage {

    private final IProjectService projectService;
    private final IProjectInstService projectInstService;
    private final ProcessInstanceService processInstanceService;
    private final ProcessModelManage processModelManage;
    private final DeclaredProjectHelper declaredProjectHelper;
    private final ProjectChangeStopHelper projectChangeStopHelper;
    private final ProjectLibManage projectLibManage;
    private final IProjectChangeHistoryService projectChangeHistoryService;

    @Transactional(rollbackFor = Exception.class)
    public synchronized void submitProjectChangeApply(SubmitProjectChangeApplyReq req) {
        ProjectEditDTO projectEdit = req.getProjectEditDetail();
        ProjectDTO reqProject = projectEdit.getProject();
        Project project = projectService.getNewProject(reqProject.getId());
        if (ProjectStatus.ON_CHANGE_APPLY.eq(project.getStatus()) ||
                ProjectStatus.STOPPED_PASSED.eq(project.getStatus())) {
            throw BizException.wrap("暂无法提交变更申请");
        }
        ProjectDetailVO projDetailOld = projectLibManage.getProjectDetail(project);

        List<FieldDiff> projDiffs = BeanDiffUtil.diffSameField(projDetailOld, reqProject,
                ProjectDTO.SYSTEM_REPLACE_INFOS);
        List<GovSystemReplaceInfoDTO> systemReplaceInfos = reqProject.getSystemReplaceInfos();
        List<GovSystemReplaceInfoDTO> systemReplaceInfosOld = projDetailOld.getSystemReplaceInfos();
        if (!CollUtil.isEqualList(systemReplaceInfos, systemReplaceInfosOld)) {
            projDiffs.add(FieldDiff.builder()
                    .fieldName("systemReplaceInfos")
                    .fileType(List.class.getTypeName())
                    .oldValue(systemReplaceInfosOld)
                    .newValue(systemReplaceInfos)
                    .build());
        }
        projectEdit.setProjectDiffs(projDiffs);
        List<PurchaseFullInfoVO> purchaseDetails = projDetailOld.getPurchaseDetails();
        if (purchaseDetails != null && !purchaseDetails.isEmpty()) {
            Map<Long, PurchaseEditDTO> reqPurchaseMap = CollUtils.listToMap(projectEdit.getPurchases(),
                    w -> w.getPurchase().getId());
            for (PurchaseFullInfoVO purchaseDetail : purchaseDetails) {
                PurchaseVO purchase = purchaseDetail.getPurchase();
                ContractVO contract = purchaseDetail.getConstruction();
                OperationVO operation = purchaseDetail.getOperation();

                PurchaseEditDTO reqPurchaseDetail = reqPurchaseMap.get(purchase.getId());
                PurchaseDTO reqPurchase = reqPurchaseDetail.getPurchase();
                OperationDTO reqOperation = reqPurchaseDetail.getOperation();
                ContractDTO reqContract = reqPurchaseDetail.getContract();

                List<FieldDiff> purchaseDiffs = BeanDiffUtil.diffSameField(purchase, reqPurchase);
                List<FieldDiff> operationDiffs = BeanDiffUtil.diffSameField(operation, reqOperation);
                List<FieldDiff> contractDiffs = BeanDiffUtil.diffSameField(contract, reqContract);
                reqPurchaseDetail.setPurchaseDiffs(purchaseDiffs);
                reqPurchaseDetail.setContractDiffs(contractDiffs);
                reqPurchaseDetail.setOperationDiffs(operationDiffs);
            }
        }

        UserInfoDetails userDetail = LoginUserUtil.loginUserDetail();

        WflowModels wflowModels = processModelManage.getWflowModelsNoNull(ProjectProcessType.PROJECT_CHANGE, userDetail.getMhUnitId());

        ProcessStartParamsVo params = new ProcessStartParamsVo();
        params.setUser(declaredProjectHelper.buildUser(userDetail));
        params.setProcessUsers(Collections.emptyMap());
        params.setFormData(Collections.emptyMap());

        // 获取发起单位、发起单位主管单位、发起单位上级主管条线单位信息
        Map<String, OrgInfoDTO> orgModelMap = declaredProjectHelper.buildOrgModelMap(userDetail, project);
        String instId = processInstanceService.startProcessLs(wflowModels, params, orgModelMap);
        ProjectInst projectInst = new ProjectInst();
        projectInst.setInstCode(instId);
        projectInst.setProjectId(project.getId());
        projectInst.setInstType(ProjectProcessType.PROJECT_CHANGE);
        projectInstService.save(projectInst);
        projectChangeStopHelper.changeProjectPurchaseStatus(project, CHANGE_APPLY_SUBMIT);

        // 保存变更历史
        ProjectChangeHistory projectChangeHistory = new ProjectChangeHistory();
        projectChangeHistory.setChangeContent(JSONUtil.toJsonStr(projectEdit));
        projectChangeHistory.setChangeReason(req.getChangeReason());
        projectChangeHistory.setProjectId(project.getId());
        projectChangeHistory.setProjectCode(project.getProjectCode());
        projectChangeHistory.setStatus(ProjectChangeStatus.ON_CHANGE_APPLY.getCode());
        projectChangeHistory.setInstCode(instId);
        projectChangeHistoryService.save(projectChangeHistory);
    }

    public PageVo<ProjectChangeListVO> page(ProjectListReq req) {
        LambdaQueryWrapper<Project> wrapper = ProjectManageUtil.projectQuery(req)
                .eq(Project::getNewest, Boolean.TRUE);
        UserInfoDetails userDetail = LoginUserUtil.loginUserDetail();
        if (!userDetail.getSuperAdmin()) {
            wrapper.eq(Project::getBuildOrgCode, userDetail.getMhUnitIdStr());
        }
        Page<ProjectChangeListDTO> page = projectService.pageChangeList(wrapper, req.page());
        if (page.getTotal() == 0) {
            return PageVo.empty();
        }
        List<ProjectChangeListDTO> records = page.getRecords();
        List<ProjectChangeListVO> data = records.stream().map(w -> {
            ProjectChangeListVO item = new ProjectChangeListVO();
            BeanUtil.copyProperties(w, item);
            item.setUnitStripName(MhUnitStripEnum.getVal(w.getUnitStrip()));
            return item;
        }).collect(Collectors.toList());
        return PageVo.of(data, page.getTotal());
    }

    public ProjectChangeDetailVO changeDetail(String projectCode, String instCode) {
        LambdaQueryWrapper<ProjectChangeHistory> wrapper = Wrappers.lambdaQuery(ProjectChangeHistory.class)
                .eq(ProjectChangeHistory::getProjectCode, projectCode)
                .eq(ProjectChangeHistory::getInstCode, instCode);
        ProjectChangeHistory changeHistory = projectChangeHistoryService.getOne(wrapper);
        if (changeHistory == null) {
            return null;
        }
        ProjectChangeDetailVO result = new ProjectChangeDetailVO();
        result.setProjectEditDetail(JSONUtil.toBean(changeHistory.getChangeContent(), ProjectEditDTO.class));
        result.setChangeReason(changeHistory.getChangeReason());
        result.setCreateOn(changeHistory.getCreateOn());
        return result;
    }

    public List<ProjectLibListItemVO> options() {
        LambdaQueryWrapper<Project> query = Wrappers.lambdaQuery(Project.class)
                .select(Project::getId, Project::getProjectCode, Project::getProjectName)
                .eq(Project::getNewest, Boolean.TRUE)
                .orderByDesc(Project::getCreateOn)
                .notIn(Project::getStatus, ProjectStatus.STOPPED.getCode(),
                        ProjectStatus.ON_STOPPED_APPLY.getCode(),
                        ProjectStatus.ON_CHANGE_APPLY);
        UserInfoDetails userDetail = LoginUserUtil.loginUserDetail();
        if (!userDetail.getSuperAdmin()) {
            query.eq(Project::getBuildOrgCode, userDetail.getMhUnitIdStr());
        }
        List<Project> projects = projectService.list(query);
        return CollUtils.convert(projects, w -> {
            ProjectLibListItemVO item = new ProjectLibListItemVO();
            item.setProjectName(w.getProjectName());
            item.setProjectCode(w.getProjectCode());
            item.setId(w.getId());
            return item;
        });
    }

}
